library douban.fm;

import 'dart:async';
import 'dart:convert';
import 'package:http/http.dart';
import './src/constants.dart';
import './src/models.dart';
export './src/models.dart' show PlayType, Channel, Song, Lyrics, User;

String _error = '';

String getErrorMsg() {
  return _error;
}

_extractError(Map data) {
  if (data['err'] != null) {
    _error = data['err'];
  }
}

class FM {
  User _user;

  Future<User> login({String account, String password}) async {
    Response response = await post(kLoginUrl, headers: {
      'Content-Type': 'application/x-www-form-urlencoded'
    }, body: {
      'email': account,
      'password': password,
      'app_name': kAppName,
      'version': kApiVersion
    });
    Map body = JSON.decode(response.body);
    if (body.containsKey('r') && body['r'] == kLoginSuccess) {
      _user = new User(
          userId: body['user_id'],
          token: body['token'],
          expire: body['expire'],
          userName: body['user_name'],
          email: body['email']);
    } else {
      _user = null;
    }
    _extractError(body);
    return _user;
  }

  Future<List<Channel>> fetchChannelList() async {
    List<Channel> result = <Channel>[];
    Response response = await get(kChannelUrl);
    Map body = JSON.decode(response.body);
    if (body.containsKey('channels')) {
      List channelList = body['channels'];
      channelList.forEach((channel) {
        result.add(new Channel.fromJson(channel));
      });
    }
    return result;
  }

  Future<List<Song>> fetchSongList(PlayType playType,
      {String userId,
      int expire,
      String token,
      String songId,
      String channelId}) async {
    Map paramMap = {
      'app_name': kAppName,
      'version': kApiVersion,
      'sid': songId,
      'channel': channelId,
      'type': playTypeMap[playType],
    };
    if (_user != null) {
      paramMap['user_id'] = _user.userId;
      paramMap['expire'] = _user.expire;
      paramMap['token'] = _user.token;
    }

    String params = '';
    paramMap.forEach((k, v) {
      if (v != null) {
        if (params.length > 0) {
          params += '&';
        }
        params += '$k=$v';
      }
    });

    List<Song> result = <Song>[];
    Response response = await get(kSongUrl + '?' + params);
    Map body = JSON.decode(response.body);
    if (body.containsKey('r') &&
        body['r'] == kLoginSuccess &&
        body.containsKey('song')) {
      List songList = body['song'];
      songList.forEach((song) {
        result.add(new Song.fromJson(song));
      });
    }
    _extractError(body);
    return result;
  }

  Future<Lyrics> fetchLyrics(String songId, String ssId) async {
    Lyrics result;
    Response response = await post(kLyricsUrl,
        headers: {'Content-Type': 'application/x-www-form-urlencoded'},
        body: {'sid': songId, 'ssid': ssId});
    Map body = JSON.decode(response.body);
    if (body.containsKey('lyric') && body.containsKey('name')) {
      result = new Lyrics.fromJson(body);
    }
    _extractError(body);
    return result;
  }
}
